@@ -15,24 +15,26 @@ var EventEmitter = require('events').EventEmitter;
 var stdio = process.binding('stdio');
 
 
-
 exports.createInterface = function (output, completer) {
   return new Interface(output, completer);
 };
 
 function Interface (output, completer) {
+  if (!(this instanceof Interface)) return new Interface(output, completer);
+  EventEmitter.call(this);
+
   this.output = output;
   this.completer = completer;
 
-  this.setPrompt("node> ");
+  this.setPrompt("> ");
 
-  this._tty = output.fd < 3;
+  this.enabled = output.fd < 3;  // Looks like a TTY.
 
   if (parseInt(process.env['NODE_NO_READLINE'])) {
-    this._tty = false;
+    this.enabled = false;
   }
 
-  if (this._tty) {
+  if (this.enabled) {
     // input refers to stdin
 
     // Current line
@@ -40,22 +42,28 @@ function Interface (output, completer) {
 
     // Check process.env.TERM ?
     stdio.setRawMode(true);
-    this._tty = true;
+    this.enabled = true;
 
     // Cursor position on the line.
     this.cursor = 0;
 
     this.history = [];
     this.historyIndex = -1;
+
+    exports.columns = process.binding('stdio').getColumns();
+
+    if (process.listeners("SIGWINCH").length === 0) {
+      process.on("SIGWINCH", function () {
+        exports.columns = process.binding('stdio').getColumns();
+      });
+    }
   }
 }
 
 inherits(Interface, EventEmitter);
 
 Interface.prototype.__defineGetter__("columns", function () {
-  if (this._tty) {
-    return stdio.getColumns();
-  }
+  return exports.columns;
 });
 
 Interface.prototype.setPrompt = function (prompt, length) {
@@ -65,7 +73,7 @@ Interface.prototype.setPrompt = function (prompt, length) {
 
 
 Interface.prototype.prompt  = function () {
-  if (this._tty) {
+  if (this.enabled) {
     this.cursor = 0;
     this._refreshLine();
   } else {
@@ -111,7 +119,7 @@ Interface.prototype._refreshLine  = function () {
 
 
 Interface.prototype.close = function (d) {
-  if (this._tty) {
+  if (this.enabled) {
     stdio.setRawMode(false);
   }
   this.emit('close');
@@ -121,7 +129,7 @@ Interface.prototype.close = function (d) {
 
 Interface.prototype.write = function (d) {
   if (this._closed) return;
-  return this._tty ? this._ttyWrite(d) : this._normalWrite(d);
+  return this.enabled ? this._ttyWrite(d) : this._normalWrite(d);
 };
 
 
@@ -261,7 +269,12 @@ Interface.prototype._ttyWrite = function (b) {
     /* ctrl+c */
     case 3:
       //process.kill(process.pid, "SIGINT");
-      this.close();
+      if (this.listeners('SIGINT').length) {
+        this.emit('SIGINT');
+      } else {
+        // default behavior, end the readline
+        this.close();
+      }
       break;
 
     case 4: // control-d, delete right or EOF
@@ -332,6 +345,17 @@ Interface.prototype._ttyWrite = function (b) {
       this._historyNext();
       break;
 
+    case 23: // control-w, delete backwards to a word boundary
+      if (this.cursor !== 0) {
+        var leading = this.line.slice(0, this.cursor);
+        var match = leading.match(/\s?((\W+|\w+)\s*)$/);
+        leading = leading.slice(0, leading.length - match[1].length);
+        this.line = leading + this.line.slice(this.cursor, this.line.length);
+        this.cursor = leading.length;
+        this._refreshLine();
+      }
+      break;
+
     case 9: // tab, completion
       if (this.completer) {
         this._tabComplete();
@@ -347,34 +371,104 @@ Interface.prototype._ttyWrite = function (b) {
       return;
 
     case 27:    /* escape sequence */
-      if (b[1] === 98 && this.cursor > 0) { // meta-b - backward word
+      var next_word, next_non_word, previous_word, previous_non_word;
 
-      } else if (b[1] === 102 && this.cursor < this.line.length) { // meta-f - forward word
+      if (b[1] === 98 && this.cursor > 0) {
+        // meta-b - backward word
+        previous_word = this.line.slice(0, this.cursor)
+              .split('').reverse().join('')
+              .search(/\w/);
+        if (previous_word !== -1) {
+          previous_non_word = this.line.slice(0, this.cursor - previous_word)
+              .split('').reverse().join('')
+              .search(/\W/);
+          if (previous_non_word !== -1) {
+            this.cursor -= previous_word + previous_non_word;
+            this._refreshLine();
+            break;
+          }
+        }
+        this.cursor = 0;
+        this._refreshLine();
 
-      } else if (b[1] === 91 && b[2] === 68) { // left arrow
+      } else if (b[1] === 102 && this.cursor < this.line.length) {
+        // meta-f - forward word
+        next_word = this.line.slice(this.cursor, this.line.length).search(/\w/);
+        if (next_word !== -1) {
+          next_non_word =
+            this.line.slice(this.cursor + next_word, this.line.length)
+                     .search(/\W/);
+          if (next_non_word !== -1) {
+            this.cursor += next_word + next_non_word;
+            this._refreshLine();
+            break;
+          }
+        }
+        this.cursor = this.line.length;
+        this._refreshLine();
+
+      } else if (b[1] === 100 && this.cursor < this.line.length) {
+        // meta-d delete forward word
+        next_word = this.line.slice(this.cursor, this.line.length).search(/\w/);
+        if (next_word !== -1) {
+          next_non_word =
+            this.line.slice(this.cursor + next_word, this.line.length)
+                     .search(/\W/);
+          if (next_non_word !== -1) {
+            this.line =
+              this.line.slice(this.cursor + next_word + next_non_word);
+            this.cursor = 0;
+            this._refreshLine();
+            break;
+          }
+        }
+        this.line = '';
+        this.cursor = 0;
+        this._refreshLine();
+
+      } else if (b[1] === 91 && b[2] === 68) {
+        // left arrow
         if (this.cursor > 0) {
           this.cursor--;
           this.output.write('\x1b[0D');
         }
-      } else if (b[1] === 91 && b[2] === 67) { // right arrow
+
+      } else if (b[1] === 91 && b[2] === 67) {
+        // right arrow
         if (this.cursor != this.line.length) {
           this.cursor++;
           this.output.write('\x1b[0C');
         }
-      } else if (b[1] === 91 && b[2] === 72) { // home
+
+      } else if ((b[1] === 91 && b[2] === 72) ||
+                 (b[1] === 79 && b[2] === 72) ||
+                 (b[1] === 91 && b[2] === 55) ||
+                 (b[1] === 91 && b[2] === 49 && (b[3] && b[3] === 126))) {
+        // home
         this.cursor = 0;
         this._refreshLine();
-      } else if (b[1] === 91 && b[2] === 70) { // end
+      } else if ((b[1] === 91 && b[2] === 70) ||
+                 (b[1] === 79 && b[2] === 70) ||
+                 (b[1] === 91 && b[2] === 56) ||
+                 (b[1] === 91 && b[2] === 52 && (b[3] && b[3] === 126))) {
+        // end
         this.cursor = this.line.length;
         this._refreshLine();
-      } else if (b[1] === 91 && b[2] === 65) { // up arrow
+
+      } else if (b[1] === 91 && b[2] === 65) {
+        // up arrow
         this._historyPrev();
-      } else if (b[1] === 91 && b[2] === 66) { // down arrow
+
+      } else if (b[1] === 91 && b[2] === 66) {
+        // down arrow
         this._historyNext();
-      } else if (b[1] === 91 && b[2] === 51 && this.cursor < this.line.length) { // delete right
+
+      } else if (b[1] === 91 && b[2] === 51 && this.cursor < this.line.length) {
+        // delete right
         this.line = this.line.slice(0, this.cursor) +
                     this.line.slice(this.cursor+1, this.line.length);
         this._refreshLine();
+
       }
       break;
 
