@@ -6,7 +6,7 @@ var package = require("./package");
 
 assert.equal("world", package.hello);
 
-debug("load fixtures/b/c.js");
+common.debug("load fixtures/b/c.js");
 
 var string = "C";
 
